/** 
 @file md-modes.js
 
 @brief This file contain implementation of methods required to store data from UI.

 @author: Gregor

 @date: 17.06.2013
*/


/** 
 @brief Creates new ID required for storing to PropertyTree for room modes
 
 @param -
  
 @return New ID value
*/
function getNewRoomModeIdValue() {
    var idValue = 0;
    var baseNode = Property.getNode('modes');
    if (baseNode == null)
        return idValue;
    var oArrayNode = baseNode.getChildren();
    if (oArrayNode != null) {
      for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
          var oChildNode = oArrayNode[iIndex];
          if (oChildNode.getChild('id') != null) {
              if (oChildNode.getChild('id').getValue() != null)
                  if (parseInt("" + oChildNode.getChild('id').getValue(),10) >= idValue)
                      idValue = parseInt("" + oChildNode.getChild('id').getValue(),10) + 1;
          }
      }
    }
    return idValue;
}

/** 
 @brief Creates new ID required for storing to PropertyTree for devices
 
 @param -
  
 @return New ID value
*/
function getNewDeviceIdValue() {
    var idValue = 0;
    var baseNode = Property.getNode('mddevices');
    if (baseNode == null)
        return idValue;
    var oArrayNode = baseNode.getChildren();
    if (oArrayNode != null) {
      for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
          var oChildNode = oArrayNode[iIndex];
          if (oChildNode.getChild('id') != null) {
              if (oChildNode.getChild('id').getValue() != null)
                  if (parseInt("" + oChildNode.getChild('id').getValue(),10) >= idValue)
                      idValue = parseInt("" + oChildNode.getChild('id').getValue(),10) + 1;
          }
      }
    }
    return idValue;
}


/** 
 @brief Saves parameter value to PropertyTree
 
 @param root  - root folder where the value will be stored 
 @param path  - sub folders where the value will be stored 
 @param value - value wich will be stored to PropertyTree
  
 @return New ID value
*/
function storeSubParameter(root, path, value) {
  Property.setProperty(root +'/' + path, value);
  Property.setFlag(root + '/' + path, 'ARCHIVE', true);
  lInfo.logln("STORING " + root + "/" + path + " -> " + value);
}

/** 
 @brief Gets the PropertyTree ID for room
 
 @param nRoomID  - ID of the searched room
  
 @return ID of room, or -1 if no room is found
*/                      
function getDefaultRoomPropertyID(nRoomID) {
   var idValue = -1;
   var baseNode = Property.getNode('modes');
   if (baseNode != null) {
     var oArrayNode = baseNode.getChildren();
     for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
       var oChildNode = oArrayNode[iIndex];
         if (oChildNode.getChild('roomID') != null) {
           if (oChildNode.getChild('roomID').getValue() != null && oChildNode.getChild('roomID').getValue() == nRoomID) {
             if (oChildNode.getChild('id').getValue() != null) {
               idValue = oChildNode.getChild('id').getValue()
               break;
             }
           }
         }
     }
   }
   return idValue;
}

/** 
 @brief Gets the PropertyTree ID for device
 
 @param sDeviceID  - ID of the searched device
  
 @return ID of device, or -1 if no device is found
*/                      
function getDefaultDevicePropertyID(sDeviceID) {
   var idValue = -1;
   var baseNode = Property.getNode('mddevices');
   if (baseNode != null) {
     var oArrayNode = baseNode.getChildren();
     for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
       var oChildNode = oArrayNode[iIndex];
         if (oChildNode.getChild('deviceID') != null) {
           if (oChildNode.getChild('deviceID').getValue() != null && oChildNode.getChild('deviceID').getValue() == sDeviceID) {
             if (oChildNode.getChild('id').getValue() != null) {
               idValue = oChildNode.getChild('id').getValue()
               break;
             }
           }
         }
     }
   }
   return idValue;
}

/** 
 @brief Gets the PropertyTree data for room
 
 @param nRoomID  - ID of the searched room
  
 @return Room data as object, or null if no room is found
*/                      
function getRoomModeData(nRoomID) {
   var roomModeData = null;
   var baseNode = Property.getNode('modes');
   if (baseNode != null) {
     var oArrayNode = baseNode.getChildren();
     for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
       var oChildNode = oArrayNode[iIndex];
         if (oChildNode.getChild('roomID') != null) {
           if (oChildNode.getChild('roomID').getValue() != null && oChildNode.getChild('roomID').getValue() == nRoomID) {
             roomModeData = oChildNode;
             break;
           }
         }
     }
   }
   return roomModeData;
}

/** 
 @brief Gets the PropertyTree data for room
 
 @param nRoomID  - ID of the searched device
  
 @return Device data as object, or null if no device is found
*/                      
function getDeviceData(sDeviceID) {
   var deviceData = null;
   var baseNode = Property.getNode('mddevices');
   if (baseNode != null) {
     var oArrayNode = baseNode.getChildren();
     for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
       var oChildNode = oArrayNode[iIndex];
         if (oChildNode.getChild('deviceID') != null) {
           if (oChildNode.getChild('deviceID').getValue() != null && oChildNode.getChild('deviceID').getValue() == sDeviceID) {
             deviceData = oChildNode;
             break;
           }
         }
     }
   }
   return deviceData;
}

/** 
 @brief Updates room mode when selecting new Sensor as Motion Detector and room has no "mode" assigned 
 
 @param sValue  - String JSON value of AKM Device data
  
 @return Returns true if new mode for room is assigned, else false
*/                      
function updateRoomModes(oObject) {
    bRefreshModesGrid = false;
    try {
        var roomID = parseInt(oObject.roomID, 10);
        var sensorFunction = parseInt(oObject.sensorFunction, 10);

        // store only if set as Motion Detector
        if (sensorFunction == 5) {
          // checking room mode
          var roomModeData = getRoomModeData(roomID);
          var roomModeID = 0;
          var bChangeRoomMode = false
          if (roomModeData == null) { // room has no "mode" stored yet
            bChangeRoomMode = true;
            roomModeID = getNewRoomModeIdValue();
            lInfo.logln("updateRoomModes newMode");
          }
          else {
            // existing roomMode, check if room Mode is set to none
            if (roomModeData.getChild('modeID').getValue() == 0) {
              bChangeRoomMode = true;
              roomModeID = roomModeData.getChild('id').getValue();
            }
            lInfo.logln("updateRoomModes2 roomMode:"+roomModeData.getChild('modeID').getValue()+" bChangeMode:"+bChangeRoomMode);
          }
          
          if (bChangeRoomMode) {
            storeSubParameter('modes', roomModeID+'/id', roomModeID);
            storeSubParameter('modes', roomModeID+'/disabled', false);
            storeSubParameter('modes', roomModeID+'/roomID', roomID);
            storeSubParameter('modes', roomModeID+'/modeID', 99); // set to advanced mode
            bRefreshModesGrid = true;
            
            lInfo.logln("updateRoomModes3 - change roomMode to advanced: "+ roomModeID);
          }
        }
    } 
    catch (e) {
        lInfo.logln("Error by saving updateRoomModes: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }
    }
    
    return bRefreshModesGrid;
}

/** 
 @brief Stores device info for grid when saving from "Device Configurator" dialog
 
 @param sValue  - String JSON value of dialog data
  
 @return 
*/                      
function storeDevice(oObject) {
  
  try { 
    var devicePropertyID = getDefaultDevicePropertyID(oObject.deviceID);
    
    if (devicePropertyID < 0)   // first time storing data for this device
      devicePropertyID = getNewDeviceIdValue()
      
    storeSubParameter('mddevices', devicePropertyID+'/id', devicePropertyID);
    storeSubParameter('mddevices', devicePropertyID+'/deviceID', oObject.deviceID);
    storeSubParameter('mddevices', devicePropertyID+'/inputMode', oObject.inputMode);
    storeSubParameter('mddevices', devicePropertyID+'/offDelay', oObject.delayTime);
      
    Property.store();
  }
  catch (e) {
      lInfo.logln("Error by saving device: " + e);
      for ( var i in e) {
          lInfo.logln(i + ': ' + e[i]);
      }
  }
}


/** 
 @brief Stores room mode when saving from "Edit Motion Detector Mode" dialog
 
 @param sValue  - String JSON value of dialog data
  
 @return 
*/                      
 function storeMode(sValue) {
    try {
        var oObject = JSON.parse(unescape(sValue));
                     
        var roomID = parseInt(oObject.roomID, 10);
        var roomPropertyID = getDefaultRoomPropertyID(roomID);
        
        var bModeDisabled = true;
        var nSelectedMode = 0;
        if (oObject.customRoomModeFields != null) 
          nSelectedMode = isNaN(parseInt(oObject.customRoomModeFields.modeID)) ? 0 : parseInt(oObject.customRoomModeFields.modeID, 10); 
        if (roomPropertyID < 0)   // first time storing data for this room
        {
          roomPropertyID = getNewRoomModeIdValue();
          lInfo.logln("bNewMode, nSelectedMode: "+ nSelectedMode+ ", bModeDisabled:"+bModeDisabled);
        }
        
        if (nSelectedMode > 0)
          bModeDisabled = false;
        
        lInfo.logln("Save room mode forModeID: "+ roomPropertyID+ ", bModeDisabled:"+bModeDisabled);
          
        // storing data to propertyTree
        storeSubParameter('modes', roomPropertyID+'/id', roomPropertyID);
        storeSubParameter('modes', roomPropertyID+'/disabled', bModeDisabled);
        storeSubParameter('modes', roomPropertyID+'/roomID', roomID);
        storeSubParameter('modes', roomPropertyID+'/modeID', nSelectedMode);
        
        // Switch off light info
        var bSwitchOff = false;
        var nSensorID = 0;
        if (oObject.customRoomModeFields != null) {
          bSwitchOff = oObject.customRoomModeFields.isDark;
          if (oObject.customRoomModeFields.twilightID != null)
            nSensorID = oObject.customRoomModeFields.twilightID;
        }
        storeSubParameter('modes', roomPropertyID+'/switchOffDarkCheck', bSwitchOff);
        storeSubParameter('modes', roomPropertyID+'/switchOffDarkSensorID', nSensorID);
        
        // Set alarm if motion was detected
        var bAlarmMotion = false;
        if (oObject.customRoomModeFields != null) 
          bAlarmMotion = oObject.customRoomModeFields.isAlarm;
        storeSubParameter('modes', roomPropertyID+'/setAlarmCheck', bAlarmMotion);
        
        // Automatically switch off
        var bAutoOff = false;
        var nAutoOffDelay = 0;
        if (oObject.customRoomModeFields != null) {
          bAutoOff = oObject.customRoomModeFields.isAutoOff;
          nAutoOffDelay = parseInt(oObject.customRoomModeFields.delayTime, 10)
        }
        storeSubParameter('modes', roomPropertyID+'/autoOffCheck', bAutoOff);
        storeSubParameter('modes', roomPropertyID+'/autoOffDelay', nAutoOffDelay);
        
        // Additional switch-off delay
        var nAdditionalOffDelay = 0;
        if (oObject.customRoomModeFields != null) 
          nAdditionalOffDelay = parseInt(oObject.customRoomModeFields.delayExtra, 10);
        storeSubParameter('modes', roomPropertyID+'/additionalOffDelayTime', nAdditionalOffDelay);  
       
        Property.store();
        
        // Creating SR
        switch(nSelectedMode){
          case 0:  // none
          case 99: // advanced mode
            MD_removeAllSceneRespondersForRoom(roomID);
          break;
         
          default:
/*
            MD_setupSceneResponder(roomID, nSelectedMode, bSwitchOff, nSensorID,
                                   bAlarmMotion, bAutoOff, nAutoOffDelay,
                                   nAdditionalOffDelay, 
                                   oObject.testMDDevice);//TESTING ONLY  */
            MD_setupSceneResponder(roomID, nSelectedMode, bSwitchOff, nSensorID,
                                   bAlarmMotion, bAutoOff, nAutoOffDelay,
                                   nAdditionalOffDelay, 
                                   0);//TESTING ONLY*/
          break;   
        }
    } 
    catch (e) {
        lInfo.logln("Error by saving storeMode: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }
    }
}

function registerMDStates() {
    try {                
      var baseNode = Property.getNode('modes');  
      if (baseNode != null) {
        var oArrayNode = baseNode.getChildren();
        if (oArrayNode != null) {
          for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
              var oChildNode = oArrayNode[iIndex];
              if (oChildNode.getChild('roomID') != null) {
                if (oChildNode.getChild('roomID').getValue() != null) {
                  var nRoomID = parseInt("" + oChildNode.getChild('roomID').getValue(),10);
                  lInfo.logln("RegisterMDStates for room: " +nRoomID);  
                  var room_manual_switch_state_name = MD_buildRoomManualSwitchStateName(nRoomID);
                  var room_fade_off_state_name = MD_buildRoomFadeOffStateName(nRoomID);
                  var room_final_off_state_name = MD_buildRoomFinalOffStateName(nRoomID);                   
                   
                  registerState(room_manual_switch_state_name, true);
                  registerState(room_fade_off_state_name, true);
                  registerState(room_final_off_state_name, true); 
                  lInfo.logln("Register state: " + room_manual_switch_state_name);
                  lInfo.logln("Register state: " + room_fade_off_state_name);
                  lInfo.logln("Register state: " + room_final_off_state_name);
               }
             }
          }
        }
      }
    }          
    catch (e) {
        lInfo.logln("Error by registerMDStates: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }
    }                  
}

if (raisedEvent.name === 'running') {
    lInfo.logln("motion-detector started");
    Property.load();
    convertPropertyTreeEntries();
    registerMDStates();
}
else
if (raisedEvent.name == 'motion-detector.mode-config') {
  if (raisedEvent.parameter.actions == 'save') {   
    storeMode(raisedEvent.parameter.value);
    var evt = new Event("motion-detector-room-mode.saved", {});
    evt.raise();
  }
  else
    if (raisedEvent.parameter.actions == 'disable') {   
     Property.load();
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
       var nRoomModeID = getDefaultRoomPropertyID(oIDObject[sKey]);
       if (nRoomModeID > -1) {  
           storeSubParameter('modes', nRoomModeID+'/disabled', true);
           // Disables all SR for this room
           MD_setAllSceneRespondersForRoomState(oIDObject[sKey], false);
       }
      }
      Property.store();
      var evt = new Event("motion-detector-room-mode.saved", {});
      evt.raise();
    }
    else
      if (raisedEvent.parameter.actions == 'enable') {  
        Property.load(); 
        lInfo.logln("Enable modeID:"+raisedEvent.parameter.id);  
        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        if (typeof(oIDObject)!='object')
            oIDObject=[oIDObject];

        for (var sKey in oIDObject) {
         var nRoomModeID = getDefaultRoomPropertyID(oIDObject[sKey]);
         if (nRoomModeID > -1) {  
             storeSubParameter('modes', nRoomModeID+'/disabled', false);
             // Enables all SR for this room
             MD_setAllSceneRespondersForRoomState(oIDObject[sKey], true);
         }
        }
        Property.store();
        var evt = new Event("motion-detector-room-mode.saved", {});
        evt.raise();
       
      }
      else
        if (raisedEvent.parameter.actions == 'deviceupdate') 
        {   
          var oObject = JSON.parse(unescape(raisedEvent.parameter.value));
          storeDevice(oObject);
          var bRefreshModesGrid = updateRoomModes(oObject);

          var evt = new Event("motion-detector-devices.saved", {});
          evt.raise();
        }
}


    
